#include <cctype>
#include <cstdio>
#include <string>
#include <vector>
#include <list>
#include <map>

using namespace std;

static bool nodata = true;

static const bool startwith(const char *s2, const string &s, unsigned ind=0)
{
    unsigned a;
    for(a=ind; s2 && *s2; ++a)
    {
        if(a >= s.size())return false;
        if(toupper(s[a]) != *s2++)
            return false;
    }
    return true;
}

static string ucase(const string &s)
{
	string t;
    unsigned a, b=s.size();
    for(a=0; a<b; ++a)t += toupper(s[a]);
    return t;
}

static string GetStr(const string &s, unsigned &a)
{
    string t;
    while(a < s.size() && (isalnum(s[a]) || s[a]=='_'))
        t += s[a++];
    return t;
}
static const bool ischar(char c)
{
    return c=='_' || isalpha(c);
}

static string sqlfix(const string &s)
{
	string t;
    unsigned a, b=s.size();
    for(a=0; a<b; ++a)
    {
        if(s[a]=='\\' || s[a]=='\'')t += '\\';
        t += s[a];
    }
    return t;
}

class fieldtype
{
    string type;
    unsigned width, prec;
    bool autoincrement;
    bool zerofill;
    bool unsig;
    bool null;
    string deftext;
    string content;
    bool binary, unique;
    void RebuildContent()
    {
        content = type;
        if(width)
        {
            char Buf[64];
            content += '(';
            sprintf(Buf, "%u", width); content += Buf;
            if(prec) { sprintf(Buf, ",%u", prec); content += Buf; }
            content += ')';
        }
        content += ' ';
        if(zerofill)content += "ZEROFILL ";
        if(unsig)content += "UNSIGNED ";
        if(binary)content += "BINARY ";
        if(unique)content += "UNIQUE ";
        if(null)content += "NULL ";else { content += "NOT NULL "; forcedefault(); }
        if(autoincrement)
        {
            content += "AUTO_INCREMENT ";
        }
    }
public:
    inline const bool operator!= (const fieldtype &s) const { return !operator==(s); }
    const bool operator== (const fieldtype &s) const
    {
        return s.width==width
           &&  s.prec==prec
           &&  s.autoincrement==autoincrement
           &&  s.zerofill==zerofill
           &&  s.unsig==unsig
           &&  s.null==null
           &&  s.type==type
           &&  s.binary==binary;
    }
    const char *c_str() const { return content.c_str(); }
    const string defval() const
    {
        if(deftext.size())
        {
            return "'" + sqlfix(deftext) + "'";
        }
        return "";
    }
    void forcedefault()
    {
        if(!deftext.size() && (type=="INT" || type=="SMALLINT" || type=="BIGINT"))
        {
            deftext = "0";
        }
    }
    const string &gettype() const { return type; }
    const unsigned getwidth() const { return width; }
    void settype(const string &t) { type=t; RebuildContent(); }
    explicit fieldtype(const string &s) : width(0), prec(0), autoincrement(false),
                                          zerofill(false), unsig(false), null(true),
                                          content(s), binary(false), unique(false)
    {
        unsigned b=s.size(), c=0;
        for(; c<b && isspace(s[c]); ++c);
        for(type=""; c<b && ischar(s[c]); ++c)type += toupper(s[c]);
        if(!type.size())
        {
            throw("Can't get type from '" + s + "'");
        }
        if(type=="TINYINT" || type=="INTEGER")type="INT";
        else if(type=="REAL")type="DOUBLE";
        else if(type=="NUMERIC")type="DECIMAL";
        if(type=="INT")width=11;
        else if(type=="SMALLINT")width=6;
        else if(type=="MEDIUMINT")width=9;
        else if(type=="BIGINT")width=20;
        else if(type=="DOUBLE")width=16, prec=4;
        else if(type=="FLOAT")width=10, prec=2;
        else if(type=="CHAR")width=1;
        for(; c<b && isspace(s[c]); ++c);
        if(c<b)
        {
            if(s[c]=='(')
            {
                for(++c; c<b && isspace(s[c]); ++c);
                for(width=0; c<b && isdigit(s[c]); ++c)width=width*10+s[c]-'0';
                for(; c<b && isspace(s[c]); ++c);
                if(s[c]==',')
                {
                    for(++c; c<b && isspace(s[c]); ++c);
                    for(prec=0; c<b && isdigit(s[c]); ++c)prec=prec*10+s[c]-'0';
                    for(; c<b && isspace(s[c]); ++c);
                }
                if(s[c]!=')')
                {
                    throw("Expected ')' in '" + s + "'");
                }
                ++c;
            }
            while(c<b)
            {
                string t;
                for(; c<b && isspace(s[c]); ++c);
                //fprintf(stderr, "Analyzing '%s'...\n", s.substr(c).c_str());
                if(c>=b)break;
                for(t=""; c<b && ischar(s[c]); ++c)t += toupper(s[c]);
                if(!t.size())
                {
                    throw("Can't get a word from '" + s + "' at '" + s.substr(c) + "'");
                }
                if(t=="AUTO_INCREMENT")autoincrement = true;
                else if(t=="UNSIGNED")unsig = true;
                else if(t=="BINARY")binary = true;
                else if(t=="UNIQUE")unique = true;
                else if(t=="ZEROFILL")zerofill = true;
                else if(t=="NULL")null = true;
                else if(t=="PRIMARY")
                {
                    for(; c<b && isspace(s[c]); ++c);
                    for(t=""; c<b && ischar(s[c]); ++c)t += toupper(s[c]);
                    if(t!="KEY")throw("PRIMARY "+t+"? KEY expected.");
                    /* ignore */
                }
                else if(t=="NOT")
                {
                    for(; c<b && isspace(s[c]); ++c);
                    for(t=""; c<b && ischar(s[c]); ++c)t += toupper(s[c]);
                    if(t!="NULL")throw("NOT "+t+"? NULL expected.");
                    null = false;
                }
                else if(t=="DEFAULT")
                {
                    for(; c<b && isspace(s[c]); ++c);
                    if(s[c]=='\'')
                    {
                        deftext = "";
                        while(c<b)
                        {
                            ++c;
                            if(s[c]=='\'')break;
                            if(s[c]=='\\')
                            {
                                ++c;
                                switch(s[c])
                                {
                                    case 'n': deftext += '\n'; break;
                                    case 'r': deftext += '\r'; break;
                                    case 't': deftext += '\t'; break;
                                    default: deftext += s[c];
                                }
                            }
                            else
                                deftext += s[c];
                        }
                        ++c;
                    }
                    else
                    {
                        for(deftext=""; c<b && !isspace(s[c]); ++c)deftext += toupper(s[c]);
                        if(deftext=="NULL")deftext="";
                    }
                }
                else
                    throw("What is '" + t + "' of '" + s + "'? Type is '" + type + "'");
            }
            //fprintf(stderr, "(Done)\n");
        }
        if(type=="INT") { if(width==11)width=0; }
        else if(type=="BIGINT") { if(width==20)width=0; }
        else if(type=="SMALLINT") { if(width==6)width=0; }
        else if(type=="DOUBLE") { if(width==16 && prec==4)width=prec=0; }
        else if(type=="FLOAT") { if(width==10 && prec==2)width=prec=0; }
        else if(type=="CHAR") { if(width==1)width=0; }
        else if(type=="VARCHAR" && width<4)type="CHAR";
        
        RebuildContent();
    }
};
typedef pair<string, fieldtype> fieldpairtype;
typedef vector<fieldpairtype> fieldlisttype;
typedef pair<string, string> keypairtype;
typedef vector<keypairtype> keylisttype;

string database = "winnie3";

class GeneerinenVirhe { public: bool pip; GeneerinenVirhe(bool p) : pip(p) { } };

static bool CaseEqual(const string &a, const string &b)
{
	unsigned aasi = a.size();
	unsigned beesi= b.size();
	if(aasi != beesi)return false;
	for(unsigned p=0; p<aasi; ++p)
		if(toupper(a[p]) != toupper(b[p]))
			return false;
	return true;
}

class Table
{
    string nam;
    string primarykey;
    
public:
    fieldlisttype fields;
    keylisttype keys;
    
    vector<string> inserts;
    
    Table(const string &name) : nam(name)
    {
        
    }
    void add(const string &name, const string &type)
    {
    //  fprintf(stderr, "*** Table '%s': '%s' = '%s'\n", nam.c_str(), name.c_str(), type.c_str());
        fields.push_back(fieldpairtype(name, fieldtype(type)));
    }
    void addkey(const string &name, const string &type)
    {
        keys.push_back(keypairtype(name, type));
    }
    void setprimary(const string &n)
    {
        primarykey = n;
        for(;;)
        {
            unsigned a = primarykey.find(", ");
            if(a==primarykey.npos)break;
            primarykey.erase(a+1, 1);
        }
        // fprintf(stderr, "*** Table '%s': primary = '%s'\n", nam.c_str(), n.c_str());
    }
    inline const string &name() const { return nam; }
    inline const string &pkey() const { return primarykey; }
    bool hasfield(const string &n) const
    {
        unsigned a, b=fields.size();
        for(a=0; a<b; ++a)
            if(CaseEqual(fields[a].first, n))return true;
        return false;
    }
    bool haskey(const string &n) const
    {
        unsigned a, b=keys.size();
        for(a=0; a<b; ++a)
            if(CaseEqual(keys[a].first, n))return true;
        return false;
    }
    const fieldpairtype &getfield(const string &n) const
    {
        unsigned a, b=fields.size();
        for(a=0; a<b; ++a)
            if(CaseEqual(fields[a].first, n))
                return fields[a];
        return fields[0]; // error
    }
    const string getkey(const string &n) const
    {
        unsigned a, b=keys.size();
        for(a=0; a<b; ++a)
            if(CaseEqual(keys[a].first, n))
                return keys[a].second;
        return "";
    }
    void Insert(const string &s)
    {
        inserts.push_back(s);
    }
    void FlushInserts() const
    {
        unsigned a, b=inserts.size();
        for(a=0; a<b; ++a)
            printf("insert into %s %s;\n", nam.c_str(), inserts[a].c_str());
    }
    inline bool tyhjataulu() const { return inserts.size() == 0; }
    void postfix()
    {
        unsigned a;
        bool hasvar = false;
        for(a=0; a<fields.size(); ++a)
        {
            if(fields[a].second.gettype()=="VARCHAR"
            || fields[a].second.gettype()=="TEXT"
            || fields[a].second.gettype()=="BLOB")
                hasvar = true;
        }
        if(hasvar)
        {
            for(a=0; a<fields.size(); ++a)
            {
                if(fields[a].second.gettype() == "CHAR"
                && fields[a].second.getwidth() > 3)
                    fields[a].second.settype("VARCHAR");
            }
        }
    }
};

typedef pair<string, Table> tabletype;
typedef map<string, Table> tablelisttype;

class sqldump
{
    tablelisttype tables;
    
    void load(const string &s) // analyzes the result
    {
        unsigned a, b=s.size();
        for(a=0; a<b; ++a)
        {
            if(s[a]=='#')
            {
                while(a+1<b && s[++a]!='\n');
                continue;
            }
            #define empty(x) ((x)=='\n' || (x)=='\r' || (x)=='\t' || (x)==' ')
            if(empty(s[a]))continue;
            //fprintf(stderr, "*** Testing: '%s'\n", s.substr(a,50).c_str());
            if(startwith("CREATE ", s, a))
            {
                #define SKIPBLANK while(a < b && empty(s[a]))++a;
                a += 7;
                SKIPBLANK
                if(startwith("INDEX ", s, a))
                {
                    throw string("Create index isn't working. Use key instead.");
                    a += 6;
                }
                if(!startwith("TABLE ", s, a))goto SkipCmd;
                a += 6;
                Table t(GetStr(s, a));
                SKIPBLANK
                if(s[a] == '(')++a;
                for(;;)
                {
                    SKIPBLANK
                    if(s[a]=='#')
                    {
                        while(a+1<b && s[++a]!='\n');
                        continue;
                    }
                    string fldname = GetStr(s, a);
                    string content;
                    SKIPBLANK
                    if(ucase(fldname) == "PRIMARY")
                    {
                        if(ucase(GetStr(s, a)) == "KEY")
                        {
                            SKIPBLANK
                            if(s[a]=='(')++a;
                            while(a < b && s[a] != ')')content += s[a++];
                            ++a;
                            t.setprimary(content);
                        }                        
                    }
                    else if(ucase(fldname) == "KEY")
                    {
                        string k;
                        SKIPBLANK
                        while(a < b && s[a]!='(' && s[a] != ',' && !isspace(s[a]))k += s[a++];
                        SKIPBLANK
                        int sku=0;
                        while(a < b)
                        {
                            if(s[a] != ' ')content += s[a];
                            if(s[a] == '(')++sku;
                            else if(s[a] == ')')
                            {
                                if(!--sku)break;
                            }
                            ++a;
                        }
                        ++a;
                        t.addkey(k, content);
                    }
                    else if(ucase(fldname) == "UNIQUE")
                    {
                    RetryUnique:
                        string k = "#";
                        SKIPBLANK
                        unsigned c=0;
                        while(a < b && (ischar(s[a]) || (c && isdigit(s[a])))) { c++; k += s[a++]; }
                        if(k=="#KEY")goto RetryUnique;
                        SKIPBLANK
                        int sku=0;
                        while(a < b)
                        {
                            if(s[a] != ' ')content += s[a];
                            if(s[a] == '(')++sku;
                            else if(s[a] == ')')
                            {
                                if(!--sku)break;
                            }
                            ++a;
                        }
                        ++a;
                        t.addkey(k, content);
                    }
                    else 
                    {
                        bool heitto = false;
                        bool empti = true;
                        unsigned sulku = 0;
                        while(a < b && (sulku || (s[a] != ',' && s[a] != ')') || heitto))
                        {    
                            if(s[a]=='#' && !heitto)
                            {
                                while(a+1<b && s[++a]!='\n');
                                continue;
                            }
                            if(empty(s[a]))
                                empti = true;
                            else
                            {
                                if(empti && content.size())content += ' ';
                                content += heitto ? s[a] : toupper(s[a]);
                                empti = false;
                            }
                            if(s[a] == '(')++sulku;
                            else if(s[a] == ')')--sulku;
                            if(s[a] == '\\')content += s[++a];
                            else if(s[a] == '\'')heitto = !heitto;
                            ++a;
                        }
                        if(content.find("PRIMARY KEY") != content.npos)
                        {
                            t.setprimary(fldname);
                        }
                        t.add(fldname, content);
                    }
                    SKIPBLANK
                    if(s[a] == ')')break;
                    ++a; /* skip , */
                }
                SKIPBLANK
                
                t.postfix();
                tables.insert(tabletype(t.name(), t));
                
                continue;
            }
            if(startwith("INSERT INTO ", s, a))
            {
                a += 11;
                SKIPBLANK;
                string tabname = GetStr(s, a);
                SKIPBLANK;
                bool heitto=false;
                string content;
                while(a < b && (s[a] != ';' || heitto))
                {
                    content += s[a];
                    if(s[a] == '\\')content += s[++a];
                    else if(s[a] == '\'')heitto = !heitto;
                    ++a;
                }
                if(!nodata)
                    tables.find(tabname)->second.Insert(content);
            }
SkipCmd:    bool heitto=false;
            while(a < b && (s[a] != ';' || heitto))
            {
                if(s[a] == '\\')++a;
                else if(s[a] == '\'')heitto = !heitto;
                ++a;
            }
        }
    }
public:
    sqldump(FILE *fp, bool ispipe=false)
    {
        if(!fp)
        {
            throw GeneerinenVirhe(ispipe);
        }
        fprintf(stderr, "*** Reading%s...\n", ispipe?" pipe":" file");
        char Buf[4096];
        string s;
        for(;;)
        {
            int n = fread(Buf, 1, sizeof Buf, fp);
            if(n <= 0)break;
            s += string(Buf, n);
        }
        if(ispipe)
            pclose(fp);
        else
            fclose(fp);
        fprintf(stderr, "*** Analyzing...\n");
        load(s);
    }
    
    void updatewith(const sqldump &uusi)
    {
        fprintf(stderr, "*** Combining...\n");
        tablelisttype::const_iterator i, j;
        
        if(!tables.size())
            printf("CREATE DATABASE %s;\n", database.c_str());
        printf("USE %s;\n", database.c_str());    
        
        for(i=tables.begin(); i!=tables.end(); ++i)
        {
            j = uusi.tables.find(i->first);
            if(j == uusi.tables.end())
            {
                printf("DROP TABLE %s;\n", i->first.c_str());
                continue;
            }
            fieldlisttype::const_iterator k;
            
            bool altertable = false;
            
            for(k = i->second.fields.begin(); k != i->second.fields.end(); ++k)
            {
                if(j->second.hasfield(k->first))
                {
                    const fieldpairtype &fp = j->second.getfield(k->first);
                    const fieldtype &f = fp.second;
                    const string olddef = k->second.defval();
                    const string newdef = f.defval();
                    if(k->second != f)
                    {
                    	if(!altertable)
                    		{printf("ALTER TABLE %s ", i->first.c_str());altertable=true;}
                    	else
                    		printf(",\n            ");
                        printf(
                            "CHANGE %s %s %s",
                                k->first.c_str(),
                                fp.first.c_str(),
                                f.c_str());
                        if(newdef.size())
                        {
                            printf(" DEFAULT %s", newdef.c_str());
                        }
                    }
                    if(olddef.size() && olddef != newdef)
                    {
                    	if(!altertable)
                    		{printf("ALTER TABLE %s ", i->first.c_str());altertable=true;}
                    	else
                    		printf(",\n            ");
                        printf("ALTER %s DROP DEFAULT",
                            k->first.c_str());
                    }
                    if(olddef != newdef && newdef.size())
                    {
                    	if(altertable)printf(";\n");
                    	printf("ALTER TABLE %s ", i->first.c_str()); altertable=true;
                        printf("ALTER %s SET DEFAULT %s;\n",
                            k->first.c_str(), newdef.c_str());
                        altertable = false;
                        printf("UPDATE %s SET %s=%s WHERE %s IS NULL;\n",
                            i->first.c_str(), k->first.c_str(), newdef.c_str(),
                            k->first.c_str());
                    }
                }
                else
                {
                	if(!altertable)
                		{printf("ALTER TABLE %s ", i->first.c_str());altertable=true;}
                	else
                		printf(",\n            ");
                    printf("DROP %s", k->first.c_str());
                    continue;
                }
            }
            keylisttype::const_iterator m;
            for(m = i->second.keys.begin(); m != i->second.keys.end(); ++m)
            {
                string kt = j->second.getkey(m->first);
                if(kt != m->second)
                {
                	const char *mkey = m->first.c_str();
                	if(!altertable)
                		{printf("ALTER TABLE %s ", i->first.c_str());altertable=true;}
                	else
                		printf(",\n            ");
                    printf("DROP INDEX %s", (mkey[0]=='#' ? mkey+1 : mkey));
                    if(kt.size())
                    {
                    	if(!altertable)
                    		{printf("ALTER TABLE %s ", i->first.c_str());altertable=true;}
                    	else
                    		printf(",\n            ");
                    	if(mkey[0]=='#')
                    		printf("ADD UNIQUE %s %s",
                    		       mkey+1,
                    		       kt.c_str());
                    	else
	                        printf("ADD INDEX %s %s",
    	                        mkey,
            	                kt.c_str());
                    }
                }
            }
            if(altertable)printf(";\n");
        }
        for(i=uusi.tables.begin(); i!=uusi.tables.end(); ++i)
        {
            bool altertable = false;
            
            j = tables.find(i->first);
            if(j == tables.end())
                printf("CREATE TABLE %s\n(\n", i->first.c_str());
            else
            {
                if(i->second.pkey().size()
                && j->second.pkey() != i->second.pkey())
                {
                	if(!altertable)
                		{printf("ALTER TABLE %s ", i->first.c_str());altertable=true;}
                	else
                		printf(",\n            ");
                    printf("DROP PRIMARY KEY");
                }
            }
            bool primarydone = false;
            fieldlisttype::const_iterator k;
            bool eka = true;
            string ed = "";
            for(k = i->second.fields.begin(); k != i->second.fields.end(); ++k)
            {
                if(j == tables.end())
                {
                    if(eka)eka=false;else printf(",\n");
                    printf("  %s %s", k->first.c_str(), k->second.c_str());
                    const string defval = k->second.defval();
                    if(defval.size())
                        printf(" DEFAULT %s", defval.c_str());
                }
                else
                {
                    if(!j->second.hasfield(k->first))
                    {
                    	if(!altertable)
                    		{printf("ALTER TABLE %s ", i->first.c_str());altertable=true;}
                    	else
                    		printf(",\n            ");

                        printf("ADD %s %s",
                            k->first.c_str(), k->second.c_str());
                            
                        const string defval = k->second.defval();
                        if(defval.size())
                            printf(" DEFAULT %s ", defval.c_str());
                        
                        if(k->first==i->second.pkey())
                        {
                            primarydone = true;
                            printf(" PRIMARY KEY ");
                        }
                        
                        if(ed.size())
                            printf(" AFTER %s ", ed.c_str());
                        else
                            printf(" FIRST ");
                    }
                }
                ed = k->first.c_str();
            }
            keylisttype::const_iterator m;
            for(m = i->second.keys.begin(); m != i->second.keys.end(); ++m)
            {
            	const char *mkey = m->first.c_str();
                if(j == tables.end())
                {
                    if(eka)eka=false;else printf(",\n");
                    printf("%s %s %s",
                    	(mkey[0]=='#' ? "UNIQUE" : "KEY"),
                        (mkey[0]=='#' ? mkey+1 : mkey),
                        m->second.c_str());
                }
                else
                {
                    string kt = j->second.getkey(m->first);
                    if(kt != m->second)
                    {
                        if(kt.size())
                        {
                        	if(!altertable)
                        		{printf("ALTER TABLE %s ", i->first.c_str());altertable=true;}
                        	else
                        		printf(",\n            ");
                            printf("DROP INDEX %s", (mkey[0]=='#' ? mkey+1 : mkey));
                        }
                        if(m->second.size())
                        {
                        	if(!altertable)
                        		{printf("ALTER TABLE %s ", i->first.c_str());altertable=true;}
                        	else
                        		printf(",\n            ");
                        	if(mkey[0]=='#')
                        		printf("ADD UNIQUE %s %s",
                        		       mkey+1,
                        		       m->second.c_str());
                        	else
    	                        printf("ADD INDEX %s %s",
        	                        mkey,
                	                m->second.c_str());
                	    }
                    }
                }
            }
            if(altertable)
                printf(";\n");
            
            if(j == tables.end())
            {
                if(i->second.pkey().size())
                {
                    if(!eka)printf(",\n");
                    printf("  PRIMARY KEY(%s)", i->second.pkey().c_str());
                    eka = false;
                }
                if(!eka)printf("\n");
                printf(");\n");
                i->second.FlushInserts();
            }
            else if(j->second.tyhjataulu())
            {
                i->second.FlushInserts();
            }
            else if(!primarydone)
            {
                if(j->second.pkey() != i->second.pkey())
                {
                    printf("ALTER TABLE %s ADD PRIMARY KEY (%s);\n",
                        j->first.c_str(), i->second.pkey().c_str());
                }
            }
        }
    }
};

static void Usage(void)
{
    printf(
        "sqlupdate - Copyright (C) Joel Yliluoma (http://iki.fi/bisqwit/)\n"
        "\n"
        "Usage:\n"
        "    sqlupdate [options] >changes.sql\n"
        "       (Creates an update script)\n"
        "\n"
        "Options:\n"
        "    -t tablefile          Describes the file containing\n"
        "                          the new sql layout. Default: tables.sql\n"
        "    -d database           Default: winnie3\n"
        "    -h host               Default: localhost\n"
        "    -u user               Default: root\n"
        "    -p pass\n"
        "    -r                    Reverse operation. (new->old)\n"
        "    -D                    Insert new data\n"
        "    uudeksi               Normal operation.\n"
        "    vanhaksi              Reverse operation.\n"
        "\n"
        "Example:\n"
        "  ./sqlupdate -t etimgr.sql -d etimgr | mysql -uroot\n"
        "\n"
        "No changes will be done. This program\n"
        "only displays the differences.\n\n");
}

int main(int argc, const char *const *argv)
{
    string uusifile = "tables.sql";
    string host = "localhost";
    string user = "root";
    string pass = ""; 
    int vanhaksi = 0;
    
    while(--argc > 0)
    {
        const char *s = *++argv;
        if(*s=='-')
            while(*++s)
                switch(*s)
                {
                    #define stringparm(svar) \
                        if(*++s){svar = s;s=" ";}else{if(!--argc){Usage();return -1;};svar=*++argv;s=" ";}
                    
                    case 't': stringparm(uusifile); break;
                    case 'd': stringparm(database); break;
                    case 'h': stringparm(host); break;
                    case 'u': stringparm(user); break;
                    case 'p': stringparm(pass); break;
                    case 'D':
                        nodata = false;
                        break;
                    case 'r':
                        vanhaksi = 1;
                        break;
                    default:
                    	Usage();
                    	return -1;
                }
        else if(string(s)=="uudeksi")
            vanhaksi = 0;
        else if(string(s)=="vanhaksi")
            vanhaksi = 1;
        else
        {
        	Usage();
            return -1;
        }
    }
    if(vanhaksi < 0) { Usage(); return -1; }
        
    string vanhakomento = "/usr/bin/mysqldump -e -u'" + user + "' -h'" + host + "'";
    if(nodata)
        vanhakomento += " -d";
    
    if(pass.size())
        vanhakomento += " -p'" + pass + "'";
    
    vanhakomento += " '";
    vanhakomento += database;
    vanhakomento += "'";
    
    try
    {
        sqldump vanha(popen(vanhakomento.c_str(), "r"), true);
        sqldump uusi(fopen(uusifile.c_str(), "r"));
    
        if(vanhaksi==1)
            uusi.updatewith(vanha);
        else
            vanha.updatewith(uusi);
    }
    catch(const GeneerinenVirhe &p)
    {
        fprintf(stderr, "*** %s failed!\n", p.pip ? database.c_str() : uusifile.c_str());
    }
    catch(const string &s)
    {
        fprintf(stderr, "*** Oops.\n*** %s\n", s.c_str());
    }
        
    fprintf(stderr, "*** Done\n");
    
    return 0;
}
